@@ -17,32 +17,25 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
+# spell-checker:ignore dwim
 """Generic utility helpers."""
+from __future__ import annotations
 
 import contextlib
 import inspect
 import logging
 import os
+import re
 import warnings
 from argparse import Namespace
-from collections.abc import ItemsView
+from collections.abc import ItemsView, Mapping
 from functools import lru_cache
 from pathlib import Path
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Generator,
-    List,
-    Optional,
-    Sequence,
-    Tuple,
-    Union,
-)
+from typing import Any, Callable, Generator, Sequence
 
 import yaml
-from ansible import constants
 from ansible.errors import AnsibleError, AnsibleParserError
+from ansible.module_utils.parsing.convert_bool import boolean
 from ansible.parsing.dataloader import DataLoader
 from ansible.parsing.mod_args import ModuleArgsParser
 from ansible.parsing.splitter import split_args
@@ -51,18 +44,7 @@ from ansible.parsing.yaml.loader import AnsibleLoader
 from ansible.parsing.yaml.objects import AnsibleBaseYAMLObject, AnsibleSequence
 from ansible.plugins.loader import add_all_plugin_dirs
 from ansible.template import Templar
-
-try:
-    from ansible.module_utils.parsing.convert_bool import boolean
-except ImportError:
-    try:
-        from ansible.utils.boolean import boolean
-    except ImportError:
-        try:
-            from ansible.utils import boolean
-        except ImportError:
-            boolean = constants.mk_boolean
-
+from ansible.utils.collection_loader import AnsibleCollectionConfig
 from yaml.composer import Composer
 from yaml.representer import RepresenterError
 
@@ -71,16 +53,28 @@ from ansiblelint._internal.rules import (
     LoadingFailureRule,
     RuntimeErrorRule,
 )
+from ansiblelint.app import get_app
 from ansiblelint.config import options
-from ansiblelint.constants import FileType
+from ansiblelint.constants import (
+    FILENAME_KEY,
+    INCLUSION_ACTION_NAMES,
+    LINE_NUMBER_KEY,
+    NESTED_TASK_KEYS,
+    PLAYBOOK_TASK_KEYWORDS,
+    ROLE_IMPORT_ACTION_NAMES,
+    SKIPPED_RULES_KEY,
+    FileType,
+)
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, discover_lintables
+from ansiblelint.skip_utils import is_nested_task
 from ansiblelint.text import removeprefix
 
 # ansible-lint doesn't need/want to know about encrypted secrets, so we pass a
 # string as the password to enable such yaml files to be opened and parsed
 # successfully.
 DEFAULT_VAULT_PASSWORD = "x"
+COLLECTION_PLAY_RE = re.compile(r"^[\w\d_]+\.[\w\d_]+\.[\w\d_]+$")
 
 PLAYBOOK_DIR = os.environ.get("ANSIBLE_PLAYBOOK_DIR", None)
 
@@ -90,17 +84,17 @@ _logger = logging.getLogger(__name__)
 
 def parse_yaml_from_file(filepath: str) -> AnsibleBaseYAMLObject:
     """Extract a decrypted YAML object from file."""
-    dl = DataLoader()
-    if hasattr(dl, "set_vault_password"):
-        dl.set_vault_password(DEFAULT_VAULT_PASSWORD)
-    return dl.load_from_file(filepath)
+    dataloader = DataLoader()
+    if hasattr(dataloader, "set_vault_password"):
+        dataloader.set_vault_password(DEFAULT_VAULT_PASSWORD)
+    return dataloader.load_from_file(filepath)
 
 
 def path_dwim(basedir: str, given: str) -> str:
     """Convert a given path do-what-I-mean style."""
-    dl = DataLoader()
-    dl.set_basedir(basedir)
-    return str(dl.path_dwim(given))
+    dataloader = DataLoader()
+    dataloader.set_basedir(basedir)
+    return str(dataloader.path_dwim(given))
 
 
 def ansible_templar(basedir: str, templatevars: Any) -> Templar:
@@ -113,67 +107,48 @@ def ansible_templar(basedir: str, templatevars: Any) -> Templar:
     if os.path.basename(basedir) == "tasks":
         basedir = os.path.dirname(basedir)
 
-    dl = DataLoader()
-    dl.set_basedir(basedir)
-    templar = Templar(dl, variables=templatevars)
+    dataloader = DataLoader()
+    dataloader.set_basedir(basedir)
+    templar = Templar(dataloader, variables=templatevars)
     return templar
 
 
 def ansible_template(
     basedir: str, varname: Any, templatevars: Any, **kwargs: Any
 ) -> Any:
-    """Render a templated string."""
+    """Render a templated string by mocking missing filters."""
     templar = ansible_templar(basedir=basedir, templatevars=templatevars)
-    return templar.template(varname, **kwargs)
+    # pylint: disable=unused-variable
+    for i in range(3):
+        try:
+            kwargs["disable_lookups"] = True
+            return templar.template(varname, **kwargs)
+        except AnsibleError as exc:
+            if (
+                "was found, however lookups were disabled from templating"
+                in exc.message
+            ):
+                # ansible core does an early exit when disable_lookup=True but
+                # this happens after the jinja2 syntax already passed.
+                break
+            if (
+                exc.message.startswith("template error while templating string:")
+                and "'" in exc.message
+            ):
+                missing_filter = exc.message.split("'")[1]
+                if missing_filter == "end of print statement":
+                    raise
+                # Mock the filter to avoid and error from Ansible templating
+                # pylint: disable=protected-access
+                templar.environment.filters._delegatee[missing_filter] = lambda x: x
+                # Record the mocked filter so we can warn the user
+                if missing_filter not in options.mock_filters:
+                    _logger.debug("Mocking missing filter %s", missing_filter)
+                    options.mock_filters.append(missing_filter)
+                continue
+            raise
 
 
-LINE_NUMBER_KEY = "__line__"
-FILENAME_KEY = "__file__"
-
-VALID_KEYS = [
-    "name",
-    "action",
-    "when",
-    "async",
-    "poll",
-    "notify",
-    "first_available_file",
-    "include",
-    "include_tasks",
-    "import_tasks",
-    "import_playbook",
-    "tags",
-    "register",
-    "ignore_errors",
-    "delegate_to",
-    "local_action",
-    "transport",
-    "remote_user",
-    "sudo",
-    "sudo_user",
-    "sudo_pass",
-    "when",
-    "connection",
-    "environment",
-    "args",
-    "any_errors_fatal",
-    "changed_when",
-    "failed_when",
-    "check_mode",
-    "delay",
-    "retries",
-    "until",
-    "su",
-    "su_user",
-    "su_pass",
-    "no_log",
-    "run_once",
-    "become",
-    "become_user",
-    "become_method",
-    FILENAME_KEY,
-]
-
 BLOCK_NAME_TO_ACTION_TYPE_MAP = {
     "tasks": "task",
     "handlers": "handler",
@@ -185,7 +160,7 @@ BLOCK_NAME_TO_ACTION_TYPE_MAP = {
 }
 
 
-def tokenize(line: str) -> Tuple[str, List[str], Dict[str, str]]:
+def tokenize(line: str) -> tuple[str, list[str], dict[str, str]]:
     """Parse a string task invocation."""
     tokens = line.lstrip().split(" ")
     if tokens[0] == "-":
@@ -194,15 +169,15 @@ def tokenize(line: str) -> Tuple[str, List[str], Dict[str, str]]:
         tokens = tokens[1:]
     command = tokens[0].replace(":", "")
 
-    args = list()
-    kwargs = dict()
-    nonkvfound = False
+    args = []
+    kwargs = {}
+    non_kv_found = False
     for arg in tokens[1:]:
-        if "=" in arg and not nonkvfound:
-            kv = arg.split("=", 1)
-            kwargs[kv[0]] = kv[1]
+        if "=" in arg and not non_kv_found:
+            key_value = arg.split("=", 1)
+            kwargs[key_value[0]] = key_value[1]
         else:
-            nonkvfound = True
+            non_kv_found = True
             args.append(arg)
     return (command, args, kwargs)
 
@@ -215,27 +190,16 @@ def _playbook_items(pb_data: AnsibleBaseYAMLObject) -> ItemsView:  # type: ignor
 
     # "if play" prevents failure if the play sequence contains None,
     # which is weird but currently allowed by Ansible
-    # https://github.com/ansible-community/ansible-lint/issues/849
+    # https://github.com/ansible/ansible-lint/issues/849
     return [item for play in pb_data if play for item in play.items()]  # type: ignore
 
 
 def _set_collections_basedir(basedir: str) -> None:
     # Sets the playbook directory as playbook_paths for the collection loader
-    try:
-        # Ansible 2.10+
-        # noqa: # pylint:disable=cyclic-import,import-outside-toplevel
-        from ansible.utils.collection_loader import AnsibleCollectionConfig
-
-        AnsibleCollectionConfig.playbook_paths = basedir
-    except ImportError:
-        # Ansible 2.8 or 2.9
-        # noqa: # pylint:disable=cyclic-import,import-outside-toplevel
-        from ansible.utils.collection_loader import set_collection_playbook_paths
-
-        set_collection_playbook_paths(basedir)
+    AnsibleCollectionConfig.playbook_paths = basedir
 
 
-def find_children(lintable: Lintable) -> List[Lintable]:  # noqa: C901
+def find_children(lintable: Lintable) -> list[Lintable]:  # noqa: C901
     """Traverse children of a single file or folder."""
     if not lintable.path.exists():
         return []
@@ -249,13 +213,13 @@ def find_children(lintable: Lintable) -> List[Lintable]:  # noqa: C901
     else:
         try:
             playbook_ds = parse_yaml_from_file(str(lintable.path))
-        except AnsibleError as e:
-            raise SystemExit(str(e))
+        except AnsibleError as exc:
+            raise SystemExit(exc) from exc
     results = []
     basedir = os.path.dirname(str(lintable.path))
     # playbook_ds can be an AnsibleUnicode string, which we consider invalid
     if isinstance(playbook_ds, str):
-        raise MatchError(filename=str(lintable.path), rule=LoadingFailureRule())
+        raise MatchError(filename=lintable, rule=LoadingFailureRule())
     for item in _playbook_items(playbook_ds):
         # if lintable.kind not in ["playbook"]:
         #     continue
@@ -267,7 +231,7 @@ def find_children(lintable: Lintable) -> List[Lintable]:  # noqa: C901
 
             # Repair incorrect paths obtained when old syntax was used, like:
             # - include: simpletask.yml tags=nginx
-            valid_tokens = list()
+            valid_tokens = []
             for token in split_args(path_str):
                 if "=" in token:
                     break
@@ -285,6 +249,7 @@ def template(
     basedir: str,
     value: Any,
     variables: Any,
+    fail_on_error: bool = False,
     fail_on_undefined: bool = False,
     **kwargs: str,
 ) -> Any:
@@ -300,26 +265,32 @@ def template(
         # I guess the filter doesn't like empty vars...
     except (AnsibleError, ValueError, RepresenterError):
         # templating failed, so just keep value as is.
-        pass
+        if fail_on_error:
+            raise
     return value
 
 
 def play_children(
-    basedir: str, item: Tuple[str, Any], parent_type: FileType, playbook_dir: str
-) -> List[Lintable]:
+    basedir: str, item: tuple[str, Any], parent_type: FileType, playbook_dir: str
+) -> list[Lintable]:
     """Flatten the traversed play tasks."""
-    delegate_map: Dict[str, Callable[[str, Any, Any, FileType], List[Lintable]]] = {
+    # pylint: disable=unused-argument
+    delegate_map: dict[str, Callable[[str, Any, Any, FileType], list[Lintable]]] = {
         "tasks": _taskshandlers_children,
         "pre_tasks": _taskshandlers_children,
         "post_tasks": _taskshandlers_children,
         "block": _taskshandlers_children,
         "include": _include_children,
+        "ansible.builtin.include": _include_children,
         "import_playbook": _include_children,
+        "ansible.builtin.import_playbook": _include_children,
         "roles": _roles_children,
         "dependencies": _roles_children,
         "handlers": _taskshandlers_children,
         "include_tasks": _include_children,
+        "ansible.builtin.include_tasks": _include_children,
         "import_tasks": _include_children,
+        "ansible.builtin.import_tasks": _include_children,
     }
     (k, v) = item
     add_all_plugin_dirs(os.path.abspath(basedir))
@@ -329,7 +300,7 @@ def play_children(
             v = template(
                 os.path.abspath(basedir),
                 v,
-                dict(playbook_dir=PLAYBOOK_DIR or os.path.abspath(basedir)),
+                {"playbook_dir": PLAYBOOK_DIR or os.path.abspath(basedir)},
                 fail_on_undefined=False,
             )
             return delegate_map[k](basedir, k, v, parent_type)
@@ -338,39 +309,51 @@ def play_children(
 
 def _include_children(
     basedir: str, k: str, v: Any, parent_type: FileType
-) -> List[Lintable]:
+) -> list[Lintable]:
     # handle special case include_tasks: name=filename.yml
-    if k == "include_tasks" and isinstance(v, dict) and "file" in v:
+    if k in INCLUSION_ACTION_NAMES and isinstance(v, dict) and "file" in v:
         v = v["file"]
 
+    # we cannot really parse any jinja2 in includes, so we ignore them
+    if not v or "{{" in v:
+        return []
+
+    if "import_playbook" in k and COLLECTION_PLAY_RE.match(v):
+        # Any import_playbooks from collections should be ignored as ansible
+        # own syntax check will handle them.
+        return []
+
     # handle include: filename.yml tags=blah
     # pylint: disable=unused-variable
-    (command, args, kwargs) = tokenize("{0}: {1}".format(k, v))
+    (command, args, kwargs) = tokenize(f"{k}: {v}")
 
     result = path_dwim(basedir, args[0])
-    if not os.path.exists(result):
-        result = path_dwim(os.path.join(os.path.dirname(basedir)), v)
+    while basedir not in ["", "/"]:
+        if os.path.exists(result):
+            break
+        basedir = os.path.dirname(basedir)
+        result = path_dwim(basedir, args[0])
+
     return [Lintable(result, kind=parent_type)]
 
 
 def _taskshandlers_children(
-    basedir: str, k: str, v: Union[None, Any], parent_type: FileType
-) -> List[Lintable]:
-    results: List[Lintable] = []
+    basedir: str, k: str, v: None | Any, parent_type: FileType
+) -> list[Lintable]:
+    results: list[Lintable] = []
     if v is None:
         raise MatchError(
             message="A malformed block was encountered while loading a block.",
             rule=RuntimeErrorRule(),
         )
-    for th in v:
-
+    for task_handler in v:
         # ignore empty tasks, `-`
-        if not th:
+        if not task_handler:
             continue
 
         with contextlib.suppress(LookupError):
             children = _get_task_handler_children_for_tasks_or_playbooks(
-                th,
+                task_handler,
                 basedir,
                 k,
                 parent_type,
@@ -378,40 +361,41 @@ def _taskshandlers_children(
             results.append(children)
             continue
 
-        if (
-            "include_role" in th or "import_role" in th
-        ):  # lgtm [py/unreachable-statement]
-            th = normalize_task_v2(th)
-            _validate_task_handler_action_for_role(th["action"])
+        if any(x in task_handler for x in ROLE_IMPORT_ACTION_NAMES):
+            # lgtm [py/unreachable-statement]
+            task_handler = normalize_task_v2(task_handler)
+            _validate_task_handler_action_for_role(task_handler["action"])
             results.extend(
                 _roles_children(
                     basedir,
                     k,
-                    [th["action"].get("name")],
+                    [task_handler["action"].get("name")],
                     parent_type,
-                    main=th["action"].get("tasks_from", "main"),
+                    main=task_handler["action"].get("tasks_from", "main"),
                 )
             )
             continue
 
-        if "block" not in th:
+        if "block" not in task_handler:
             continue
 
-        results.extend(_taskshandlers_children(basedir, k, th["block"], parent_type))
-        if "rescue" in th:
+        results.extend(
+            _taskshandlers_children(basedir, k, task_handler["block"], parent_type)
+        )
+        if "rescue" in task_handler:
             results.extend(
-                _taskshandlers_children(basedir, k, th["rescue"], parent_type)
+                _taskshandlers_children(basedir, k, task_handler["rescue"], parent_type)
             )
-        if "always" in th:
+        if "always" in task_handler:
             results.extend(
-                _taskshandlers_children(basedir, k, th["always"], parent_type)
+                _taskshandlers_children(basedir, k, task_handler["always"], parent_type)
             )
 
     return results
 
 
 def _get_task_handler_children_for_tasks_or_playbooks(
-    task_handler: Dict[str, Any],
+    task_handler: dict[str, Any],
     basedir: str,
     k: Any,
     parent_type: FileType,
@@ -419,26 +403,31 @@ def _get_task_handler_children_for_tasks_or_playbooks(
     """Try to get children of taskhandler for include/import tasks/playbooks."""
     child_type = k if parent_type == "playbook" else parent_type
 
-    task_include_keys = "include", "include_tasks", "import_playbook", "import_tasks"
-    for task_handler_key in task_include_keys:
-
+    # Include the FQCN task names as this happens before normalize
+    for task_handler_key in INCLUSION_ACTION_NAMES:
         with contextlib.suppress(KeyError):
-
             # ignore empty tasks
-            if not task_handler:
+            if not task_handler:  # pragma: no branch
                 continue
 
-            # import pdb; pdb.set_trace()
-            return Lintable(
-                path_dwim(basedir, task_handler[task_handler_key]), kind=child_type
-            )
+            file_name = task_handler[task_handler_key]
+            if isinstance(file_name, Mapping) and file_name.get("file", None):
+                file_name = file_name["file"]
+
+            f = path_dwim(basedir, file_name)
+            while basedir not in ["", "/"]:
+                if os.path.exists(f):
+                    break
+                basedir = os.path.dirname(basedir)
+                f = path_dwim(basedir, file_name)
+            return Lintable(f, kind=child_type)
 
     raise LookupError(
-        f'The node contains none of: {", ".join(task_include_keys)}',
+        f'The node contains none of: {", ".join(sorted(INCLUSION_ACTION_NAMES))}',
     )
 
 
-def _validate_task_handler_action_for_role(th_action: Dict[str, Any]) -> None:
+def _validate_task_handler_action_for_role(th_action: dict[str, Any]) -> None:
     """Verify that the task handler action is valid for role include."""
     module = th_action["__ansible_module__"]
 
@@ -453,8 +442,12 @@ def _validate_task_handler_action_for_role(th_action: Dict[str, Any]) -> None:
 
 def _roles_children(
     basedir: str, k: str, v: Sequence[Any], parent_type: FileType, main: str = "main"
-) -> List[Lintable]:
-    results: List[Lintable] = []
+) -> list[Lintable]:
+    # pylint: disable=unused-argument # parent_type)
+    results: list[Lintable] = []
+    if not v:
+        # typing does not prevent junk from being passed in
+        return results
     for role in v:
         if isinstance(role, dict):
             if "role" in role or "name" in role:
@@ -466,15 +459,14 @@ def _roles_children(
                     )
             elif k != "dependencies":
                 raise SystemExit(
-                    'role dict {0} does not contain a "role" '
-                    'or "name" key'.format(role)
+                    f'role dict {role} does not contain a "role" or "name" key'
                 )
         else:
             results.extend(_look_for_role_files(basedir, role, main=main))
     return results
 
 
-def _rolepath(basedir: str, role: str) -> Optional[str]:
+def _rolepath(basedir: str, role: str) -> str | None:
     role_path = None
 
     possible_paths = [
@@ -488,32 +480,29 @@ def _rolepath(basedir: str, role: str) -> Optional[str]:
         path_dwim(basedir, os.path.join("..", role)),
     ]
 
-    if constants.DEFAULT_ROLES_PATH:
-        search_locations = constants.DEFAULT_ROLES_PATH
-        if isinstance(search_locations, str):
-            search_locations = search_locations.split(os.pathsep)
-        for loc in search_locations:
-            loc = os.path.expanduser(loc)
-            possible_paths.append(path_dwim(loc, role))
+    for loc in get_app().runtime.config.default_roles_path:
+        loc = os.path.expanduser(loc)
+        possible_paths.append(path_dwim(loc, role))
 
     possible_paths.append(path_dwim(basedir, ""))
 
-    for path_option in possible_paths:
+    for path_option in possible_paths:  # pragma: no branch
         if os.path.isdir(path_option):
             role_path = path_option
             break
 
-    if role_path:
+    if role_path:  # pragma: no branch
         add_all_plugin_dirs(role_path)
 
     return role_path
 
 
 def _look_for_role_files(
-    basedir: str, role: str, main: Optional[str] = "main"
-) -> List[Lintable]:
+    basedir: str, role: str, main: str | None = "main"
+) -> list[Lintable]:
+    # pylint: disable=unused-argument # main
     role_path = _rolepath(basedir, role)
-    if not role_path:
+    if not role_path:  # pragma: no branch
         return []
 
     results = []
@@ -524,18 +513,17 @@ def _look_for_role_files(
             for file in files:
                 file_ignorecase = file.lower()
                 if file_ignorecase.endswith((".yml", ".yaml")):
-                    thpath = os.path.join(folder, file)
-                    results.append(Lintable(thpath))
+                    results.append(Lintable(os.path.join(folder, file)))
 
     return results
 
 
-def _kv_to_dict(v: str) -> Dict[str, Any]:
+def _kv_to_dict(v: str) -> dict[str, Any]:
     (command, args, kwargs) = tokenize(v)
-    return dict(__ansible_module__=command, __ansible_arguments__=args, **kwargs)
+    return {"__ansible_module__": command, "__ansible_arguments__": args, **kwargs}
 
 
-def _sanitize_task(task: Dict[str, Any]) -> Dict[str, Any]:
+def _sanitize_task(task: dict[str, Any]) -> dict[str, Any]:
     """Return a stripped-off task structure compatible with new Ansible.
 
     This helper takes a copy of the incoming task and drops
@@ -544,26 +532,54 @@ def _sanitize_task(task: Dict[str, Any]) -> Dict[str, Any]:
     result = task.copy()
     # task is an AnsibleMapping which inherits from OrderedDict, so we need
     # to use `del` to remove unwanted keys.
-    for k in ["skipped_rules", FILENAME_KEY, LINE_NUMBER_KEY]:
+    for k in [SKIPPED_RULES_KEY, FILENAME_KEY, LINE_NUMBER_KEY]:
         if k in result:
             del result[k]
     return result
 
 
-def normalize_task_v2(task: Dict[str, Any]) -> Dict[str, Any]:
+def _extract_ansible_parsed_keys_from_task(
+    result: dict[str, Any],
+    task: dict[str, Any],
+    keys: tuple[str, ...],
+) -> dict[str, Any]:
+    """Return a dict with existing key in task."""
+    for k, v in list(task.items()):
+        if k in keys:
+            # we don't want to re-assign these values, which were
+            # determined by the ModuleArgsParser() above
+            continue
+        result[k] = v
+    return result
+
+
+def normalize_task_v2(task: dict[str, Any]) -> dict[str, Any]:
     """Ensure tasks have a normalized action key and strings are converted to python objects."""
-    result = dict()
+    result: dict[str, Any] = {}
+    ansible_parsed_keys = ("action", "local_action", "args", "delegate_to")
+
+    if is_nested_task(task):
+        _extract_ansible_parsed_keys_from_task(result, task, ansible_parsed_keys)
+        # Add dummy action for block/always/rescue statements
+        result["action"] = {
+            "__ansible_module__": "block/always/rescue",
+            "__ansible_module_original__": "block/always/rescue",
+        }
+
+        return result
 
     sanitized_task = _sanitize_task(task)
     mod_arg_parser = ModuleArgsParser(sanitized_task)
+
     try:
         action, arguments, result["delegate_to"] = mod_arg_parser.parse(
             skip_action_validation=options.skip_action_validation
         )
-    except AnsibleParserError as e:
+    except AnsibleParserError as exc:
+        # pylint: disable=raise-missing-from
         raise MatchError(
             rule=AnsibleParserErrorRule(),
-            message=e.message,
+            message=exc.message,
             filename=task.get(FILENAME_KEY, "Unknown"),
             linenumber=task.get(LINE_NUMBER_KEY, 0),
         )
@@ -573,30 +589,30 @@ def normalize_task_v2(task: Dict[str, Any]) -> Dict[str, Any]:
         action = "shell"
         del arguments["_uses_shell"]
 
-    for (k, v) in list(task.items()):
-        if k in ("action", "local_action", "args", "delegate_to") or k == action:
-            # we don't want to re-assign these values, which were
-            # determined by the ModuleArgsParser() above
-            continue
-        result[k] = v
+    _extract_ansible_parsed_keys_from_task(
+        result, task, ansible_parsed_keys + (action,)
+    )
 
     if not isinstance(action, str):
-        raise RuntimeError("Task actions can only be strings, got %s" % action)
+        raise RuntimeError(f"Task actions can only be strings, got {action}")
     action_unnormalized = action
     # convert builtin fqn calls to short forms because most rules know only
     # about short calls but in the future we may switch the normalization to do
     # the opposite. Mainly we currently consider normalized the module listing
     # used by `ansible-doc -t module -l 2>/dev/null`
     action = removeprefix(action, "ansible.builtin.")
-    result["action"] = dict(
-        __ansible_module__=action, __ansible_module_original__=action_unnormalized
-    )
+    result["action"] = {
+        "__ansible_module__": action,
+        "__ansible_module_original__": action_unnormalized,
+    }
 
     if "_raw_params" in arguments:
-        result["action"]["__ansible_arguments__"] = arguments["_raw_params"].split(" ")
+        # Doing a split here is really bad as it would break jinja2 templating
+        # parsing of the template must happen before any kind of split.
+        result["action"]["__ansible_arguments__"] = [arguments["_raw_params"]]
         del arguments["_raw_params"]
     else:
-        result["action"]["__ansible_arguments__"] = list()
+        result["action"]["__ansible_arguments__"] = []
 
     if "argv" in arguments and not result["action"]["__ansible_arguments__"]:
         result["action"]["__ansible_arguments__"] = arguments["argv"]
@@ -606,7 +622,7 @@ def normalize_task_v2(task: Dict[str, Any]) -> Dict[str, Any]:
     return result
 
 
-def normalize_task(task: Dict[str, Any], filename: str) -> Dict[str, Any]:
+def normalize_task(task: dict[str, Any], filename: str) -> dict[str, Any]:
     """Unify task-like object structures."""
     ansible_action_type = task.get("__ansible_action_type__", "task")
     if "__ansible_action_type__" in task:
@@ -617,7 +633,7 @@ def normalize_task(task: Dict[str, Any], filename: str) -> Dict[str, Any]:
     return task
 
 
-def task_to_str(task: Dict[str, Any]) -> str:
+def task_to_str(task: dict[str, Any]) -> str:
     """Make a string identifier for the given task."""
     name = task.get("name")
     if name:
@@ -625,30 +641,30 @@ def task_to_str(task: Dict[str, Any]) -> str:
     action = task.get("action")
     if isinstance(action, str) or not isinstance(action, dict):
         return str(action)
-    args = " ".join(
-        [
-            "{0}={1}".format(k, v)
-            for (k, v) in action.items()
-            if k
-            not in [
-                "__ansible_module__",
-                "__ansible_module_original__",
-                "__ansible_arguments__",
-                LINE_NUMBER_KEY,
-                FILENAME_KEY,
-            ]
+    args = [
+        f"{k}={v}"
+        for (k, v) in action.items()
+        if k
+        not in [
+            "__ansible_module__",
+            "__ansible_module_original__",
+            "__ansible_arguments__",
+            LINE_NUMBER_KEY,
+            FILENAME_KEY,
         ]
-    )
+    ]
+
     for item in action.get("__ansible_arguments__", []):
-        args += f" {item}"
-    return "{0} {1}".format(action["__ansible_module__"], args)
+        args.append(str(item))
+
+    return f"{action['__ansible_module__']} {' '.join(args)}"
 
 
 def extract_from_list(
-    blocks: AnsibleBaseYAMLObject, candidates: List[str], recursive: bool = False
-) -> List[Any]:
+    blocks: AnsibleBaseYAMLObject, candidates: list[str], recursive: bool = False
+) -> list[Any]:
     """Get action tasks from block structures."""
-    results = list()
+    results = []
     for block in blocks:
         for candidate in candidates:
             if isinstance(block, dict) and candidate in block:
@@ -661,15 +677,14 @@ def extract_from_list(
                     results.extend(subresults)
                 elif block[candidate] is not None:
                     raise RuntimeError(
-                        "Key '%s' defined, but bad value: '%s'"
-                        % (candidate, str(block[candidate]))
+                        f"Key '{candidate}' defined, but bad value: '{str(block[candidate])}'"
                     )
     return results
 
 
-def add_action_type(actions: AnsibleBaseYAMLObject, action_type: str) -> List[Any]:
+def add_action_type(actions: AnsibleBaseYAMLObject, action_type: str) -> list[Any]:
     """Add action markers to task objects."""
-    results = list()
+    results = []
     for action in actions:
         # ignore empty task
         if not action:
@@ -679,41 +694,29 @@ def add_action_type(actions: AnsibleBaseYAMLObject, action_type: str) -> List[An
     return results
 
 
-def get_action_tasks(yaml: AnsibleBaseYAMLObject, file: Lintable) -> List[Any]:
+def get_action_tasks(data: AnsibleBaseYAMLObject, file: Lintable) -> list[Any]:
     """Get a flattened list of action tasks from the file."""
-    tasks = list()
+    tasks = []
     if file.kind in ["tasks", "handlers"]:
-        tasks = add_action_type(yaml, file.kind)
+        tasks = add_action_type(data, file.kind)
     else:
-        tasks.extend(
-            extract_from_list(yaml, ["tasks", "handlers", "pre_tasks", "post_tasks"])
-        )
+        tasks.extend(extract_from_list(data, PLAYBOOK_TASK_KEYWORDS))
 
     # Add sub-elements of block/rescue/always to tasks list
-    tasks.extend(
-        extract_from_list(tasks, ["block", "rescue", "always"], recursive=True)
-    )
-    # Remove block/rescue/always elements from tasks list
-    block_rescue_always = ("block", "rescue", "always")
-    tasks[:] = [
-        task for task in tasks if all(k not in task for k in block_rescue_always)
-    ]
+    tasks.extend(extract_from_list(tasks, NESTED_TASK_KEYS, recursive=True))
 
-    return [
-        task
-        for task in tasks
-        if set(
-            ["include", "include_tasks", "import_playbook", "import_tasks"]
-        ).isdisjoint(task.keys())
-    ]
+    return tasks
 
 
-@lru_cache(maxsize=128)
-def parse_yaml_linenumbers(lintable: Lintable) -> AnsibleBaseYAMLObject:
+@lru_cache(maxsize=None)
+def parse_yaml_linenumbers(  # noqa: max-complexity: 12
+    lintable: Lintable,
+) -> AnsibleBaseYAMLObject:
     """Parse yaml as ansible.utils.parse_yaml but with linenumbers.
 
     The line numbers are stored in each node's LINE_NUMBER_KEY key.
     """
+    result = []
 
     def compose_node(parent: yaml.nodes.Node, index: int) -> yaml.nodes.Node:
         # the line number where the previous token has ended (plus empty lines)
@@ -731,7 +734,9 @@ def parse_yaml_linenumbers(lintable: Lintable) -> AnsibleBaseYAMLObject:
         if hasattr(node, "__line__"):
             mapping[LINE_NUMBER_KEY] = node.__line__
         else:
-            mapping[LINE_NUMBER_KEY] = mapping._line_number
+            mapping[
+                LINE_NUMBER_KEY
+            ] = mapping._line_number  # pylint: disable=protected-access
         mapping[FILENAME_KEY] = lintable.path
         return mapping
 
@@ -742,32 +747,47 @@ def parse_yaml_linenumbers(lintable: Lintable) -> AnsibleBaseYAMLObject:
         loader = AnsibleLoader(lintable.content, **kwargs)
         loader.compose_node = compose_node
         loader.construct_mapping = construct_mapping
-        data = loader.get_single_data()
-    except (yaml.parser.ParserError, yaml.scanner.ScannerError) as e:
-        logging.exception(e)
-        raise SystemExit("Failed to parse YAML in %s: %s" % (lintable.path, str(e)))
-    return data
+        # while Ansible only accepts single documents, we also need to load
+        # multi-documents, as we attempt to load any YAML file, not only
+        # Ansible managed ones.
+        while True:
+            data = loader.get_data()
+            if data is None:
+                break
+            result.append(data)
+    except (
+        yaml.parser.ParserError,
+        yaml.scanner.ScannerError,
+        yaml.constructor.ConstructorError,
+    ) as exc:
+        raise RuntimeError("Failed to load YAML file") from exc
+
+    if len(result) == 0:
+        return None  # empty documents
+    if len(result) == 1:
+        return result[0]
+    return result
 
 
-def get_first_cmd_arg(task: Dict[str, Any]) -> Any:
+def get_first_cmd_arg(task: dict[str, Any]) -> Any:
     """Extract the first arg from a cmd task."""
     try:
         if "cmd" in task["action"]:
             first_cmd_arg = task["action"]["cmd"].split()[0]
         else:
-            first_cmd_arg = task["action"]["__ansible_arguments__"][0]
+            first_cmd_arg = task["action"]["__ansible_arguments__"][0].split()[0]
     except IndexError:
         return None
     return first_cmd_arg
 
 
-def get_second_cmd_arg(task: Dict[str, Any]) -> Any:
+def get_second_cmd_arg(task: dict[str, Any]) -> Any:
     """Extract the second arg from a cmd task."""
     try:
         if "cmd" in task["action"]:
             second_cmd_arg = task["action"]["cmd"].split()[1]
         else:
-            second_cmd_arg = task["action"]["__ansible_arguments__"][1]
+            second_cmd_arg = task["action"]["__ansible_arguments__"][0].split()[1]
     except IndexError:
         return None
     return second_cmd_arg
@@ -798,9 +818,9 @@ def is_playbook(filename: str) -> bool:
 
     try:
         f = parse_yaml_from_file(filename)
-    except Exception as e:
+    except Exception as exc:  # pylint: disable=broad-except
         _logger.warning(
-            "Failed to load %s with %s, assuming is not a playbook.", filename, e
+            "Failed to load %s with %s, assuming is not a playbook.", filename, exc
         )
     else:
         if (
@@ -814,41 +834,35 @@ def is_playbook(filename: str) -> bool:
 
 # pylint: disable=too-many-statements
 def get_lintables(
-    options: Namespace = Namespace(), args: Optional[List[str]] = None
-) -> List[Lintable]:
+    opts: Namespace = Namespace(), args: list[str] | None = None
+) -> list[Lintable]:
     """Detect files and directories that are lintable."""
-    lintables: List[Lintable] = []
+    lintables: list[Lintable] = []
 
     # passing args bypass auto-detection mode
     if args:
         for arg in args:
             lintable = Lintable(arg)
-            if lintable.kind in ("yaml", None):
-                _logger.warning(
-                    "Overriding detected file kind '%s' with 'playbook' "
-                    "for given positional argument: %s",
-                    lintable.kind,
-                    arg,
-                )
-                lintable = Lintable(arg, kind="playbook")
             lintables.append(lintable)
     else:
-
-        for filename in discover_lintables(options):
-
-            p = Path(filename)
+        for filename in discover_lintables(opts):
+            path = Path(filename)
             # skip exclusions
             try:
-                for file_path in options.exclude_paths:
-                    if str(p.resolve()).startswith(str(file_path)):
+                for file_path in opts.exclude_paths:
+                    if str(path.resolve()).startswith(str(file_path)):
                         raise FileNotFoundError(
-                            f"File {file_path} matched exclusion entry: {p}"
+                            f"File {file_path} matched exclusion entry: {path}"
                         )
-            except FileNotFoundError as e:
-                _logger.debug("Ignored %s due to: %s", p, e)
+            except FileNotFoundError as exc:
+                _logger.debug("Ignored %s due to: %s", path, exc)
                 continue
 
-            lintables.append(Lintable(p))
+            if path.is_symlink() and not path.exists():
+                _logger.warning("Ignored broken symlink %s -> %s", path, path.resolve())
+                continue
+
+            lintables.append(Lintable(path))
 
         # stage 2: guess roles from current lintables, as there is no unique
         # file that must be present in any kind of role.
@@ -857,7 +871,7 @@ def get_lintables(
     return lintables
 
 
-def _extend_with_roles(lintables: List[Lintable]) -> None:
+def _extend_with_roles(lintables: list[Lintable]) -> None:
     """Detect roles among lintables and adds them to the list."""
     for lintable in lintables:
         parts = lintable.path.parent.parts
@@ -865,7 +879,7 @@ def _extend_with_roles(lintables: List[Lintable]) -> None:
             role = lintable.path
             while role.parent.name != "roles" and role.name:
                 role = role.parent
-            if role.exists and not role.is_file():
+            if role.exists() and not role.is_file():
                 lintable = Lintable(role, kind="role")
                 if lintable not in lintables:
                     _logger.debug("Added role: %s", lintable)
@@ -878,8 +892,8 @@ def convert_to_boolean(value: Any) -> bool:
 
 
 def nested_items(
-    data: Union[Dict[Any, Any], List[Any]], parent: str = ""
-) -> Generator[Tuple[Any, Any, str], None, None]:
+    data: dict[Any, Any] | list[Any], parent: str = ""
+) -> Generator[tuple[Any, Any, str], None, None]:
     """Iterate a nested data structure."""
     warnings.warn(
         "Call to deprecated function ansiblelint.utils.nested_items. "
@@ -890,10 +904,11 @@ def nested_items(
     if isinstance(data, dict):
         for k, v in data.items():
             yield k, v, parent
-            for k, v, p in nested_items(v, k):
-                yield k, v, p
+            # pylint: disable=redefined-outer-name
+            for k, v, returned_parent in nested_items(v, k):
+                yield k, v, returned_parent
     if isinstance(data, list):
         for item in data:
             yield "list-item", item, parent
-            for k, v, p in nested_items(item):
-                yield k, v, p
+            for k, v, returned_parent in nested_items(item):
+                yield k, v, returned_parent
